@@ -41,13 +41,15 @@ policy <- function(x) x$solution$policy
 
 .policy_MDP_from_POMDP <- function(x) {
   pg <- x$solution$pg
-  bs <- x$observation_prob[['*']]
+  
+  ## all observation_probs should be the same!
+  bs <- x$observation_prob[[1L]]
   
   # create a list ith epochs
   lapply(
     seq_along(pg),
     FUN = function(i) {
-      rew <- reward(x, belief = bs, epoch = i)
+      rew <- reward_node_action(x, belief = bs, epoch = i)
       data.frame(
         state = colnames(bs),
         U = rew$reward,
