@@ -66,7 +66,7 @@
 #'
 #' **Time-dependent POMDPs:** Time dependence of transition probabilities,
 #' observation probabilities and reward structure can be modeled by considering
-#' a set of episodes representing epoch with the same settings. In the scared
+#' a set of episodes representing epochs with the same settings. In the scared
 #' tiger example (see Examples section), the tiger has the normal behavior for
 #' the first three epochs (episode 1) and then becomes scared with different
 #' transition probabilities for the next three epochs (episode 2). The episodes
@@ -116,8 +116,9 @@
 #' - `converged` did the solution converge?
 #' - `initial_belief` used initial beliefs.
 #' - `total_expected_reward` reward from the initial beliefs.
-#' - `pg`, `initial_pg_node` a list representing the policy graph. A converged solution has
-#' only a single list elements.
+#' - `pg`, `initial_pg_node` a list representing the policy graph. The epochs are 
+#'  the list entries. A converged infinite-horizon solution has
+#' only a single list elements. Finite-horizon solutions may converge early resulting in a shorter list.
 #' - `belief_states` used belief states.
 #' - `alpha` value function as hyperplanes representing the nodes in the policy graph.
 #' - `policy` the policy.
@@ -498,22 +499,31 @@ solve_POMDP <- function(model,
           cat("Convergence: Finite-horizon POMDP converged early at epoch:",
             i  -  1,
             "\n")
-        converged <- i  -  1
+        converged <- TRUE
+        
+        # we only need to keep the first pg element with the graph
+        pg <- tail(pg, n = 1L)
+        alpha <- tail(alpha, n = 1L)
+        
         break
       }
     }
+   
+    ## make transitions in last epoch NA for non converged solutions
+    if (!converged)
+      pg[[1L]][, as.character(model$observations)] <- NA 
     
+    ## order by epoch
+    alpha <- rev(alpha)
+    pg <- rev(pg)
     
     if (method == "grid" &&
         !converged &&
         any(unlist(reward_matrix(model)) < 0))
       warning(
-        "The grid method for finite horizon did not converge. The value function and the calculated reward values may not be valid with negative reward in the reward matrix. Use method 'incprune' instead."
+        "The grid method for finite horizon did not converge. The value function and the calculated reward values may not be valid with negative reward in the reward matrix. Use method 'simulate_POMDP()' to estimate the reward or use solution method 'incprune'."
       )
     
-    alpha <- rev(alpha)
-    pg <- rev(pg)
-    
   }
   
   # read belief states if available (method: grid)
@@ -543,16 +553,16 @@ solve_POMDP <- function(model,
   )
   
   ## add initial node and reward
-  rew <- reward(model, belief = model$start)
+  rew <- reward_node_action(model, belief = model$start)
   model$solution$initial_belief <- rew$belief
   model$solution$total_expected_reward <- rew$reward
   model$solution$initial_pg_node <- rew$pg_node
   
   model$solver_output <- structure(solver_output, class = "text")
- 
+  
   if(inherits(model, "MDP")) 
     model$solution$policy <- .policy_MDP_from_POMDP(model)
-   
+  
   model
 }
 
